#!/bin/bash
# example script (for country='China' and network_name='OTHER')
# converting old station_coordinates to PostGIS POINT structure
#
# new design of stationmeta_core:
#                                                  Table "public.stationmeta_core"
#              Column               |           Type           | Collation | Nullable |                   Default                                                                                         
#-----------------------------------+--------------------------+-----------+----------+----------------------------------------------                                                                     
# id                           | integer                  |           | not null | nextval('stationmeta_core_id_seq'::regclass)
# codes                        | character varying(16)[]  |           |          |
# name                         | character varying(128)   |           | not null |
# coordinates                  | geometry(PointZ,4326)    |           | not null |
# country                      | character varying(128)   |           | not null |
# state                        | character varying(128)   |           | not null |
# coordinate_validation_status | integer                  |           | not null |
# coordinate_validation_date   | timestamp with time zone |           | not null |
# type_of_environment          | integer                  |           | not null |
# type_of_area                 | integer                  |           | not null |
# category                     | character varying(128)   |           | not null |
# timezone                     | character varying(64)    |           | not null |
# additional_metadata          | jsonb                    |           | not null |
# coordinate_validator_id      | integer                  |           | not null |
#
# new (not available in old DB):
# - coordinate_validation_date (now set to NOW())
# - station_coordinate_validator_id (now set to 1 (superuser: sschroeder))
#
# author: s.schroeder@fz-juelich.de
# date:   2020-06-05

psql -h zam10131 -U s.schroeder surface_observations_toar -W -c "\COPY (select numid,station_id,station_name,station_country,station_state,station_coordinate_status,station_lon,station_lat,station_alt from stations WHERE station_country='China' and network_name='OTHER') TO 'stations_China_20200605_export.txt'"

cut -c1-427 stations_China_20200103_export.txt | sed -e "s/-/0/g" | sed -e "s/$/	NOW()	1	SRID=4326;POINT(/g" > tmp_head.txt
cut -c430- stations_China_20200103_export.txt | sed -e "s/	/ /g" | sed -e "s/$/)/g" > tmp_tail.txt
paste -d'\0' tmp_head.txt tmp_tail.txt > stations_China_20200605_import.txt
rm tmp_*.txt

psql -h localhost -U django toar2 -W -c "\COPY stationmeta_core FROM 'stations_China_20200605_import.txt'"
psql -h localhost -U django toar2 -W -c "UPDATE stationmeta_core SET country=TRIM(country),codes=TRIM(codes),name=TRIM(name),state=TRIM(state);"
