# coding: utf-8
"""
class Contact(Base)
===================
"""

from toardb.base import Base

from .models_person import Person
from .models_organisation import Organisation

from sqlalchemy import Table, Column, Integer, String, \
                       ForeignKey, text, CheckConstraint, Sequence
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship

# controlled vocabulary

# Kind of Organizations
OK_enum_table = Table("ok_vocabulary",
                Base.metadata,
                Column("enum_val", Integer, primary_key=True),
                Column("enum_str", String),
                Column("enum_display_str", String)
          )
# The following code is just a workaround (see stationmeta/models.py):
from collections import namedtuple
Enumdict=namedtuple("Dict",["value","string","display_str"])
OK_enum = (
    Enumdict(1, 'Government', 'government'),
    Enumdict(2, 'Research', 'research'),
    Enumdict(3, 'International', 'international'),
    Enumdict(4, 'NonProfit', 'non-profit'),
    Enumdict(5, 'Commercial', 'commercial'),
    Enumdict(6, 'Individual', 'individual'),
    Enumdict(7, 'Other', 'other')
    )

CONTACTS_ID_SEQ = Sequence('contacts_id_seq')  # define sequence explicitly
class Contact(Base):
    """ Table "public.contacts"

    +-----------------+---------+-----------+----------+----------------------------------------+
    |     Column      |  Type   | Collation | Nullable |                Default                 |
    +=================+=========+===========+==========+========================================+
    | id              | integer |           | not null | nextval('contacts_id_seq'::regclass)   |
    +-----------------+---------+-----------+----------+----------------------------------------+
    | person_id       | integer |           |          |                                        |
    +-----------------+---------+-----------+----------+----------------------------------------+
    | organisation_id | integer |           |          |                                        |
    +-----------------+---------+-----------+----------+----------------------------------------+
    Indexes:
        "contacts_pkey" PRIMARY KEY, btree (id)
    Check constraints:
        "ck_contact" CHECK (person_id > 0 AND organisation_id = 0 OR person_id = 0 AND organisation_id > 0)
    Foreign-key constraints:
        "contacts_organisation_id_fkey" FOREIGN KEY (organisation_id) REFERENCES organisations(id)
        "contacts_person_id_fkey" FOREIGN KEY (person_id) REFERENCES persons(id)
    """

    __tablename__ = 'contacts'
    __table_args__ = (
        CheckConstraint('((person_id > 0) AND (organisation_id = 0)) OR ((person_id = 0) AND (organisation_id > 0))'),
    )

    id = Column(Integer, primary_key=True, server_default=text("nextval('contacts_id_seq'::regclass)"))
    person_id = Column(ForeignKey('persons.id'))
    organisation_id = Column(ForeignKey('organisations.id'))

    organisation = relationship('Organisation')
    person = relationship('Person')

