"""
Pydantic schemas for TOAR database

"""

from typing import List, Dict, Any

from pydantic import BaseModel, validator
from .models import OK_enum

class OrganisationBase(BaseModel):
    id: int = None
    name: str
    longname: str
    kind: str
    city: str
    postcode: str
    street_address: str
    country: str
    homepage: str

    @validator('kind')
    def check_kind(cls, v):
        return tuple(filter(lambda x: x.value == int(v), OK_enum))[0].string


class OrganisationCreate(OrganisationBase):
    kind: str
    pass

    @validator('kind') 
    def check_kind(cls, v):
        if tuple(filter(lambda x: x.string == v, OK_enum)):
            return v
        else:
            raise ValueError(f"kind of organisation not known: {v}")


class Organisation(OrganisationBase):
    id: int

    class Config:
        orm_mode = True


class PersonBase(BaseModel):
    id: int = None
    name: str
    email: str
    phone: str
    isprivate: bool


class PersonCreate(PersonBase):
    pass


class Person(PersonBase):
    id: int

    class Config:
        orm_mode = True

# ======== for nested view =========

class Contact(BaseModel):
    person: Person
    organisation: Organisation

    class Config:
        orm_mode = True

