"""
Simple test API for stationmeta management
"""

from typing import List
from fastapi import APIRouter, Depends, HTTPException, Body
from sqlalchemy.orm import Session, sessionmaker
from . import crud, schemas
from toardb.utils.database import get_db

router = APIRouter()

# plain views to post and get stationmeta

#get all entries of table stationmeta
@router.get('/stationmeta_core/', response_model=List[schemas.StationmetaCore])
def get_all_stationmeta_core(skip: int = 0, limit: int = None, db: Session = Depends(get_db)):
    stationmeta_core = crud.get_all_stationmeta_core(db, skip=skip, limit=limit)
    return stationmeta_core

# the same as above, but nested view
#get all entries of table stationmeta
@router.get('/stationmeta/', response_model=List[schemas.Stationmeta])
def get_all_stationmeta(skip: int = 0, limit: int = None, db: Session = Depends(get_db)):
    stationmeta = crud.get_all_stationmeta(db, skip=skip, limit=limit)
    return stationmeta

#get all core metadata of one station
@router.get('/stationmeta_core/{station_code}', response_model=schemas.StationmetaCore)
def get_stationmeta_core(station_code: str, db: Session = Depends(get_db)):
    db_stationmeta_core = crud.get_stationmeta_core(db, station_code=station_code)
    if db_stationmeta_core is None:
        raise HTTPException(status_code=404, detail="Data not found.")
    return db_stationmeta_core

# the same as above, but nested view
#get all core metadata of one station
@router.get('/stationmeta/{station_code}', response_model=schemas.Stationmeta)
def get_stationmeta(station_code: str, db: Session = Depends(get_db)):
    db_stationmeta = crud.get_stationmeta(db, station_code=station_code)
    if db_stationmeta is None:
        raise HTTPException(status_code=404, detail="Data not found.")
    return db_stationmeta

#some more gets to be tested:
# - get stationmeta_global
# - get stationmeta_aux
# - ...

@router.post('/stationmeta/', response_model=schemas.Stationmeta)
# The following command was not working as long as the upload via Body was defined.
# See bug report: https://github.com/tiangolo/fastapi/issues/300
# (Although this seems to be fixed in the meantime, it is not working in my FastAPI version.)
#def create_stationmeta_core(stationmeta_core: schemas.StationmetaCoreCreate, db: Session = Depends(get_db)):
def create_stationmeta_core(stationmeta: schemas.StationmetaCreate = Body(..., embed = True), db: Session = Depends(get_db)):
    # for the  moment, just check the first code of station's codes
    db_stationmeta_core= crud.get_stationmeta_core(db, station_code=stationmeta.codes[0])
    if db_stationmeta_core:
        raise HTTPException(status_code=400, detail="Station already registered.")
    return crud.create_stationmeta(db=db, stationmeta=stationmeta)
